/* EOAccessAdditions.h created by lindberg on Mon 20-Dec-1999 */

#import <EOAccess/EOModel.h>
#import <EOAccess/EOEntity.h>
#import <EOAccess/EORelationship.h>
#import <EOControl/EOSortOrdering.h>
#import "SystemType.h"

/* EOAccess extension methods for use by templates */

@interface EOModel (EOReporterAdditions)

// Returns back the list of entities that are defined as concrete.  IE: those
// whose class is not EOGenericRecord
- (NSArray *)concreteEntityNames;
- (NSArray *)concreteEntities;

// Returns back the list of entities that are defined as generic.  IE: those
// whose class is EOGenericRecord
- (NSArray *)genericEntityNames;
- (NSArray *)genericEntities;

- (NSArray *)prototypeEntities;
- (NSArray *)entitiesMinusPrototypeEntities;
- (NSArray *)entitiesSortedByType;

@end

@interface EOEntity (EOReporterAdditions)

- (NSDictionary *)storedProcedures;
- (EOStoredProcedure *)storedProcedureForFetchAll;
- (EOStoredProcedure *)storedProcedureForFetchWithPrimaryKey;
- (EOStoredProcedure *)storedProcedureForInsert;
- (EOStoredProcedure *)storedProcedureForDelete;
- (EOStoredProcedure *)storedProcedureForPrimaryKeyGeneration;

- (BOOL)hasReadFormatOnAnAttribute;
- (BOOL)hasWriteFormatOnAnAttribute;
- (BOOL)hasPrototypeOnAnAttribute;
- (BOOL)hasDefinitionOnAnAttribute;
- (BOOL)hasValueTypeOnAnAttribute;
- (BOOL)hasDefinitionOnARelationship;
- (NSArray *)namedProperties;

- (BOOL)hasFetchLimitFetchSpecification;
- (BOOL)hasPrefetchingFetchSpecification;
- (BOOL)hasDeepFetchSpecification;
- (BOOL)hasDistinctFetchSpecification;
- (BOOL)hasLockAllFetchSpecification;
- (BOOL)hasRefreshObjectsFetchSpecification;
- (BOOL)hasRequireBindingsFetchSpecification;
- (BOOL)hasLimitPromptFetchSpecification;
- (BOOL)hasRawRowFetchSpecification;

- (BOOL)isPrototypeEntity;
- (BOOL)isPrototypeEntityInModelGroup;

@end

// This interface just insures we don't have a compile problem
@interface EOEntity (EOReporterAdditions2)
- (NSArray *)clientClassProperties;
@end

@interface EOAttribute (EOReporterAdditions)

- (BOOL)isPrimaryKey;
- (BOOL)isClassProperty;
- (BOOL)isClientClassProperty;
- (BOOL)isSnapshotProperty;

- (NSString *)fullExternalType;
- (NSString *)parameterDirectionString;

#ifdef EOF2_ONLY
// Define for use by EOF2 systems
- (NSString *)prototypeName;
- (id)prototype;
#endif

@end

@interface EORelationship (EOReporterAdditions)

- (BOOL)isClassProperty;
- (BOOL)isClientClassProperty;
- (NSString *)deleteRuleString;

@end

@interface EOFetchSpecification (EOReporterAdditions)

- (NSArray *)namedOptions;

#ifdef EOF2_ONLY
// Define for use on EOF2 systems
- (int)fetchLimit;
- (NSArray *)prefetchingRelationshipKeyPaths;
- (BOOL)requiresAllQualifierBindingVariables;
- (BOOL)promptsAfterFetchLimit;
- (BOOL)fetchesRawRows;
#endif

@end

@interface EOSortOrdering (EOReporterAdditions)

- (BOOL)isAscending;
- (BOOL)isCaseInsensitiveAscending;
- (BOOL)isDescending;
- (BOOL)isCaseInsensitiveDescending;

@end
