/* EOReporterAdditions.m created by lindberg on Mon 20-Dec-1999 */
/*-
 * Copyright (c) 2002 Doug McClure
 *
 * Original code from eogenerator Copyright (c) 2002 Carl Lindberg and Mike Gentry
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#import "EOReporterAdditions.h"
#import "EOAccessAdditions.h"
#import "FoundationAdditions.h"
#import <EOModeler/EOModelExtensions.h>

@implementation EOModel (EOReporterAdditions)

- (BOOL)hasParentInheritance
{
    NSEnumerator *enumerator = [[self entities] objectEnumerator];
    EOEntity *entity;

    while ( entity = [enumerator nextObject] ) {
        if ( [entity parentEntity] != nil ) {
            return YES;
        }
    }

    return NO;
}

- (BOOL)hasChildInheritance
{
    NSEnumerator *enumerator = [[self entities] objectEnumerator];
    EOEntity *entity;

    while ( entity = [enumerator nextObject] ) {
        if ( [[entity subEntities] count] > 0 ) {
            return YES;
        }
    }

    return NO;
}

- (BOOL)hasAbstractEntity
{
    NSEnumerator *enumerator = [[self entities] objectEnumerator];
    EOEntity *entity;

    while ( entity = [enumerator nextObject] ) {
        if ( [entity isAbstractEntity] ) {
            return YES;
        }
    }

    return NO;
}

- (BOOL)hasReadOnlyEntity
{
    NSEnumerator *enumerator = [[self entities] objectEnumerator];
    EOEntity *entity;

    while ( entity = [enumerator nextObject] ) {
        if ( [entity isReadOnly] ) {
            return YES;
        }
    }

    return NO;
}

- (NSArray *)concreteEntityNames
{
    return [self valueForKeyPath:@"concreteEntities.name"];
}

- (NSArray *)concreteEntities
{
    NSEnumerator *enumerator = [[self entities] objectEnumerator];
    NSMutableArray *entities = [NSMutableArray array];
    EOEntity *entity;

    while ( entity = [enumerator nextObject] )
        if ( ![[entity classNameWithoutPackage] isEqualToString:@"EOGenericRecord"] )
            [entities addObject:entity];

    return entities;
}

- (NSArray *)genericEntityNames
{
    return [self valueForKeyPath:@"genericEntities.name"];
}

- (NSArray *)genericEntities
{
    NSEnumerator *enumerator = [[self entities] objectEnumerator];
    NSMutableArray *entities = [NSMutableArray array];
    EOEntity *entity;

    while ( entity = [enumerator nextObject] )
        if ( [[entity classNameWithoutPackage] isEqualToString:@"EOGenericRecord"] )
            [entities addObject:entity];

    return entities;
}

- (NSArray *)prototypeEntities
{
    NSEnumerator *enumerator = [[self entities] objectEnumerator];
    NSMutableArray *entities = [NSMutableArray array];
    EOEntity *entity;

    while ( entity = [enumerator nextObject] )
        if ( [entity isPrototypeEntity] )
            [entities addObject:entity];

    return entities;
}

- (NSArray *)entitiesMinusPrototypeEntities
{
    NSEnumerator *enumerator = [[self entities] objectEnumerator];
    NSMutableArray *entities = [NSMutableArray array];
    EOEntity *entity;

    while ( entity = [enumerator nextObject] )
        if ( ![entity isPrototypeEntity] )
            [entities addObject:entity];

    return entities;
}

- (NSArray *)entitiesSortedByType
{
    NSArray *prototypes = [self prototypeEntities];

    if ( prototypes == nil )
        return [self entities];
    else
        return [prototypes arrayByAddingObjectsFromArray:[self entitiesMinusPrototypeEntities]];
}

@end


@implementation EOEntity (EOReporterAdditions)

- (NSDictionary *)storedProcedures {
    return _storedProcedures;
}

- (EOStoredProcedure *)storedProcedureForFetchAll {
    return [self storedProcedureForOperation:EOFetchAllProcedureOperation];
}

- (EOStoredProcedure *)storedProcedureForFetchWithPrimaryKey {
    return [self storedProcedureForOperation:EOFetchWithPrimaryKeyProcedureOperation];
}

- (EOStoredProcedure *)storedProcedureForInsert {
    return [self storedProcedureForOperation:EOInsertProcedureOperation];
}

- (EOStoredProcedure *)storedProcedureForDelete {
    return [self storedProcedureForOperation:EODeleteProcedureOperation];
}

- (EOStoredProcedure *)storedProcedureForPrimaryKeyGeneration {
    return [self storedProcedureForOperation:EONextPrimaryKeyProcedureOperation];
}

- (BOOL)hasReadFormatOnAnAttribute {
    NSEnumerator *enumerator = [[self attributes] objectEnumerator];
    EOAttribute *anAttr;

    while ( anAttr = [enumerator nextObject] ) {
        if ( [anAttr readFormat] != nil ) {
            return YES;
        }
    }

    return NO;
}

- (BOOL)hasWriteFormatOnAnAttribute {
    NSEnumerator *enumerator = [[self attributes] objectEnumerator];
    EOAttribute *anAttr;

    while ( anAttr = [enumerator nextObject] ) {
        if ( [anAttr writeFormat] != nil ) {
            return YES;
        }
    }

    return NO;
}

- (BOOL)hasPrototypeOnAnAttribute {
    NSEnumerator *enumerator = [[self attributes] objectEnumerator];
    EOAttribute *anAttr;

    while ( anAttr = [enumerator nextObject] ) {
        if ( [anAttr prototypeName] != nil ) {
            return YES;
        }
    }

    return NO;
}

- (BOOL)hasDefinitionOnAnAttribute {
    NSEnumerator *enumerator = [[self attributes] objectEnumerator];
    EOAttribute *anAttr;

    while ( anAttr = [enumerator nextObject] ) {
        if ( [anAttr definition] != nil ) {
            return YES;
        }
    }

    return NO;
}

- (BOOL)hasValueTypeOnAnAttribute {
    NSEnumerator *enumerator = [[self attributes] objectEnumerator];
    EOAttribute *anAttr;

    while ( anAttr = [enumerator nextObject] ) {
        if ( [anAttr valueType] != nil ) {
            return YES;
        }
    }

    return NO;
}


- (BOOL)hasDefinitionOnARelationship {
    NSEnumerator *enumerator = [[self relationships] objectEnumerator];
    EORelationship *aRelationship;

    while ( aRelationship = [enumerator nextObject] ) {
        if ( [aRelationship definition] != nil ) {
            return YES;
        }
    }

    return NO;
}

- (NSArray *)namedProperties
{
    NSMutableArray *properties = [NSMutableArray array];
    if ( [self isReadOnly] )
        [properties addObject:@"Read Only"];
    if ( [self isAbstractEntity] )
        [properties addObject:@"Abstract"];
    if ( [self cachesObjects] )
        [properties addObject:@"Caches Objects"];
    return properties;
}


- (BOOL)hasFetchLimitFetchSpecification {
    NSEnumerator *enumerator = [[self fetchSpecificationNames] objectEnumerator];
    NSString *fetchSpecName;

    while ( fetchSpecName = [enumerator nextObject] ) {
        EOFetchSpecification *fetchSpec = [self fetchSpecificationNamed:fetchSpecName];
        if ( [fetchSpec fetchLimit] > 0 )
            return YES;
    }

    return NO;
}

- (BOOL)hasPrefetchingFetchSpecification {
    NSEnumerator *enumerator = [[self fetchSpecificationNames] objectEnumerator];
    NSString *fetchSpecName;

    while ( fetchSpecName = [enumerator nextObject] ) {
        EOFetchSpecification *fetchSpec = [self fetchSpecificationNamed:fetchSpecName];
        if ( [[fetchSpec prefetchingRelationshipKeyPaths] count] > 0 )
            return YES;
    }

    return NO;
}

- (BOOL)hasDeepFetchSpecification {
    NSEnumerator *enumerator = [[self fetchSpecificationNames] objectEnumerator];
    NSString *fetchSpecName;

    while ( fetchSpecName = [enumerator nextObject] ) {
        EOFetchSpecification *fetchSpec = [self fetchSpecificationNamed:fetchSpecName];
        if ( [fetchSpec isDeep] )
            return YES;
    }

    return NO;
}


- (BOOL)hasDistinctFetchSpecification {
    NSEnumerator *enumerator = [[self fetchSpecificationNames] objectEnumerator];
    NSString *fetchSpecName;

    while ( fetchSpecName = [enumerator nextObject] ) {
        EOFetchSpecification *fetchSpec = [self fetchSpecificationNamed:fetchSpecName];
        if ( [fetchSpec usesDistinct] )
            return YES;
    }

    return NO;
}


- (BOOL)hasLockAllFetchSpecification {
    NSEnumerator *enumerator = [[self fetchSpecificationNames] objectEnumerator];
    NSString *fetchSpecName;

    while ( fetchSpecName = [enumerator nextObject] ) {
        EOFetchSpecification *fetchSpec = [self fetchSpecificationNamed:fetchSpecName];
        if ( [fetchSpec locksObjects] )
            return YES;
    }

    return NO;
}


- (BOOL)hasRefreshObjectsFetchSpecification {
    NSEnumerator *enumerator = [[self fetchSpecificationNames] objectEnumerator];
    NSString *fetchSpecName;

    while ( fetchSpecName = [enumerator nextObject] ) {
        EOFetchSpecification *fetchSpec = [self fetchSpecificationNamed:fetchSpecName];
        if ( [fetchSpec refreshesRefetchedObjects] )
            return YES;
    }

    return NO;
}


- (BOOL)hasRequireBindingsFetchSpecification {
    NSEnumerator *enumerator = [[self fetchSpecificationNames] objectEnumerator];
    NSString *fetchSpecName;

    while ( fetchSpecName = [enumerator nextObject] ) {
        EOFetchSpecification *fetchSpec = [self fetchSpecificationNamed:fetchSpecName];
        if ( [fetchSpec requiresAllQualifierBindingVariables] )
            return YES;
    }

    return NO;
}


- (BOOL)hasLimitPromptFetchSpecification {
    NSEnumerator *enumerator = [[self fetchSpecificationNames] objectEnumerator];
    NSString *fetchSpecName;

    while ( fetchSpecName = [enumerator nextObject] ) {
        EOFetchSpecification *fetchSpec = [self fetchSpecificationNamed:fetchSpecName];
        if ( [fetchSpec promptsAfterFetchLimit] )
            return YES;
    }

    return NO;
}


- (BOOL)hasRawRowFetchSpecification {
    NSEnumerator *enumerator = [[self fetchSpecificationNames] objectEnumerator];
    NSString *fetchSpecName;

    while ( fetchSpecName = [enumerator nextObject] ) {
        EOFetchSpecification *fetchSpec = [self fetchSpecificationNamed:fetchSpecName];
        if ( [fetchSpec fetchesRawRows] )
            return YES;
    }

    return NO;
}

- (BOOL)isPrototypeEntity {
    return [[self name] hasPrefix:@"EO"] && [[self name] hasSuffix:@"Prototypes"];
}

- (BOOL)isPrototypeEntityInModelGroup {
    return [[[[self model] modelGroup] models] containsObject:[self model]];
}

@end


@implementation EOAttribute (EOReporterAdditions)

- (BOOL)isPrimaryKey
{
    EOEntity *parentEntity = [self entity];
    return [[parentEntity primaryKeyAttributes] containsObject:self];
}

- (BOOL)isClassProperty
{
    EOEntity *parentEntity = [self entity];
    return [[parentEntity classProperties] containsObject:self];
}

- (BOOL)isClientClassProperty
{
    EOEntity *parentEntity = [self entity];
    return [[parentEntity clientClassProperties] containsObject:[self name]];
}

- (BOOL)isSnapshotProperty
{
    EOEntity *parentEntity = [self entity];
    return [[parentEntity attributesUsedForLocking] containsObject:self];
}

- (NSString *)fullExternalType
{
    NSString *type = [self externalType];
    unsigned precision = [self precision];
    unsigned width     = [self width];
    int      scale     = [self scale];

    if (precision && scale)
        return [NSString stringWithFormat:@"%@(%d,%d)", type, precision, scale];
    if (width || precision)
        return [NSString stringWithFormat:@"%@(%d)", type, width?width:precision];

    return type;
}

- (NSString *)parameterDirectionString
{
    switch ( [self parameterDirection] ) {
        case EOVoid:
            return @"void";
        case EOInParameter:
            return @"in";
        case EOOutParameter:
            return @"out";
        case EOInOutParameter:
            return @"in-out";
    }

    return @"";
}


#ifdef EOF2_ONLY
// Define for use by EOF2 systems

- (NSString *)prototypeName
{
    return nil;
}

- (id)prototype
{
    return nil;
}

#endif

@end


@implementation EORelationship (EOReporterAdditions)

- (BOOL)isClassProperty
{
    EOEntity *parentEntity = [self entity];
    return [[parentEntity classProperties] containsObject:self];
}

- (BOOL)isClientClassProperty
{
    EOEntity *parentEntity = [self entity];
    return [[parentEntity clientClassProperties] containsObject:self];
}

- (NSString *)deleteRuleString
{
    switch ([self deleteRule]) {
        case EODeleteRuleNullify:
            return @"Nullify";
            break;
        case EODeleteRuleCascade:
            return @"Cascade";
            break;
        case EODeleteRuleDeny:
            return @"Deny";
            break;
        case EODeleteRuleNoAction:
            return @"No Action";
    }

    return @"";
}

@end


@implementation EOFetchSpecification (EOReporterAdditions)

#ifdef EOF2_ONLY
// Define for use on EOF2 systems
- (int)fetchLimit
{
    return [[[self hints] objectForKey:EOFetchLimitHintKey] unsignedIntValue];
}
- (BOOL)promptsAfterFetchLimit
{
    return [[[self hints] objectForKey:EOPromptAfterFetchLimitHintKey] boolValue];
}
- (BOOL)requiresAllQualifierBindingVariables
{
    return NO;
}
- (BOOL)fetchesRawRows
{
    return NO;
}
- (NSArray *)prefetchingRelationshipKeyPaths
{
    return [NSArray array];
}
#endif

- (NSArray *)namedOptions
{
    NSMutableArray *namedOptions = [NSMutableArray array];

    if ( [self isDeep] )
        [namedOptions addObject:@"Perform Deep Inheritance Fetch"];
    if ( [self usesDistinct] )
        [namedOptions addObject:@"Fetch Distinct Rows"];
    if ( [self locksObjects] )
        [namedOptions addObject:@"Lock All Fetched Objects"];
    if ( [self refreshesRefetchedObjects] )
        [namedOptions addObject:@"Refresh Refectched Objects"];
    if ( [self requiresAllQualifierBindingVariables] )
        [namedOptions addObject:@"Requires All Variable Bindings"];
    if ( [self promptsAfterFetchLimit] )
        [namedOptions addObject:@"Prompt After Fetch Limit"];
    if ( [self fetchesRawRows] )
        [namedOptions addObject:@"Fetch Raw Rows"];

    return namedOptions;
}

@end



@implementation EOSortOrdering (EOReporterAdditions)

- (BOOL)isAscending {
    return [self selector] == EOCompareAscending;
}

- (BOOL)isCaseInsensitiveAscending {
    return [self selector] == EOCompareDescending;
}

- (BOOL)isDescending {
    return [self selector] == EOCompareCaseInsensitiveAscending;
}

- (BOOL)isCaseInsensitiveDescending {
    return [self selector] == EOCompareCaseInsensitiveDescending;
}

@end

